/*****************************************************************/
/*                                                               */
/*   LifeGame                                                    */
/*                                                               */
/*   Auteur       LePhenixNoir                                   */
/*   Date         10.06.2014                                     */
/*   Description  A simple simulator of Conway's game of life    */
/*                                                               */
/*   Copyright (c) 2006 CASIO COMPUTER CO., LTD.                 */
/*                                                               */
/*****************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include <EasyInput.h>
#include <PixEdit.h>
#include "fxlib.h"

#define KEY_CHAR_0 71
#define KEY_CHAR_1 72
#define KEY_CHAR_2 62
#define KEY_CHAR_3 52
#define KEY_CHAR_4 73
#define KEY_CHAR_5 63
#define KEY_CHAR_6 53
#define KEY_CHAR_7 74
#define KEY_CHAR_8 64
#define KEY_CHAR_9 54
#define KEY_CHAR_DP 61
#define KEY_CHAR_EXP 51
#define KEY_CHAR_PMINUS 41
#define KEY_CHAR_PLUS 42
#define KEY_CHAR_MINUS 32
#define KEY_CHAR_MULT 43
#define KEY_CHAR_DIV 33
#define KEY_CHAR_FRAC 75
#define KEY_CHAR_LPAR 55
#define KEY_CHAR_RPAR 45
#define KEY_CHAR_COMMA 35
#define KEY_CHAR_STORE 25
#define KEY_CHAR_LOG 66
#define KEY_CHAR_LN 56
#define KEY_CHAR_SIN 46
#define KEY_CHAR_COS 36
#define KEY_CHAR_TAN 26
#define KEY_CHAR_SQUARE 67
#define KEY_CHAR_POW 57
#define KEY_CTRL_EXE 31
#define KEY_CTRL_DEL 44
#define KEY_CTRL_AC 32
#define KEY_CTRL_FD 65
#define KEY_CTRL_EXIT 47
#define KEY_CTRL_SHIFT 78
#define KEY_CTRL_ALPHA 77
#define KEY_CTRL_OPTN 68
#define KEY_CTRL_VARS 58
#define KEY_CTRL_UP 28
#define KEY_CTRL_DOWN 37
#define KEY_CTRL_LEFT 38
#define KEY_CTRL_RIGHT 27
#define KEY_CTRL_F1 79
#define KEY_CTRL_F2 69
#define KEY_CTRL_F3 59
#define KEY_CTRL_F4 49
#define KEY_CTRL_F5 39
#define KEY_CTRL_F6 29
#define KEY_CTRL_MENU 48
#ifndef OS2Change
#define OS2Change
#ifndef OS2Change_GetOS2
#define OS2Change_GetOS2
typedef int(*sc_i2cp2sip)(char*, char*, short int*, short int*);
const unsigned int sc0015[] = { 0xD201D002, 0x422B0009, 0x80010070, 0x0015 };
#define GlibGetOSVersionInfo (*(sc_i2cp2sip)sc0015)
int OSVersionAsInt(void)
{
    unsigned char mainversion;
    unsigned char minorversion;
    unsigned short release;
    unsigned short build;
    GlibGetOSVersionInfo( &mainversion, &minorversion, &release, &build );
    return ( ( mainversion << 24 ) & 0xFF000000 ) | ( ( minorversion << 16 ) & 0x00FF0000 ) | ( release & 0x0000FFFF );
}
#define isOS2 (OSVersionAsInt() >= 0x02020000)
#define OS2(x,y) ((OSVersionAsInt() >= 0x02020000)?y:x)
#endif
#ifndef OS2Change_Keyboard
#define OS2Change_Keyboard
void delay(void)
{
    char i;
    for (i=0; i<5; i++){};
}
unsigned char CheckKeyRow(unsigned char code)
{
    unsigned char result=0;
    short*PORTB_CTRL=(void*)0xA4000102;
    short*PORTM_CTRL=(void*)0xA4000118;
    char*PORTB=(void*)0xA4000122;
    char*PORTM=(void*)0xA4000138;
    char*PORTA=(void*)0xA4000120;
    short smask;
    char cmask;
    unsigned char column, row;
    column = code>>4;
    row = code &0x0F;
    smask = 0x0003 << (( row %8)*2);
    cmask = ~( 1 << ( row %8) );
    if(row <8)
    {
        *PORTB_CTRL = 0xAAAA ^ smask;
        *PORTM_CTRL = (*PORTM_CTRL & 0xFF00 ) | 0x00AA;
        delay();
        *PORTB = cmask;
        *PORTM = (*PORTM & 0xF0 ) | 0x0F;
    }
    else
    {
        *PORTB_CTRL = 0xAAAA;
        *PORTM_CTRL = ((*PORTM_CTRL & 0xFF00 ) | 0x00AA)  ^ smask;
        delay();
        *PORTB = 0xFF;
        *PORTM = (*PORTM & 0xF0 ) | cmask;
    }
    
    delay();
    result = (~(*PORTA))>>column & 1;
    delay();
    *PORTB_CTRL = 0xAAAA;
    *PORTM_CTRL = (*PORTM_CTRL & 0xFF00 ) | 0x00AA;
    delay();
    *PORTB_CTRL = 0x5555;
    *PORTM_CTRL = (*PORTM_CTRL & 0xFF00 ) | 0x0055;
    delay();
    
    return result;
}

unsigned char KeyDown(unsigned char keycode)
{
    unsigned short key[8];
    const unsigned short* keyboardregister = (unsigned short*)0xA44B0000;
    if(isOS2)
    {
        unsigned char row = keycode%10;
        memcpy(key, keyboardregister, sizeof(unsigned short) << 3);
        
        return (0 != (key[row >> 1] & 1 << keycode / 10 - 1 + ((row & 1) << 3)));
    }
    else
    {
        return CheckKeyRow((keycode % 10) + ((keycode / 10 - 1) << 4));
    }
}
unsigned char GetKeyMod(unsigned int *key)
{
    unsigned char x, ret;
    
    ret = GetKey(key);
    
    for(x = 0; x < 80; x++)
    {
        if(KeyDown(x))
        {
            *key = x;
            break;
        }
    }
    return ret;
}
#define IsKeyDown(x) KeyDown(x)
#define IsKeyUp(x) !KeyDown(x)
#define GetKey(x) GetKeyMod(x)
#endif
#endif

void Menu   (void);
void Config (int *, int *, unsigned char *);
int  getNum (void);
void Launch (int,int,unsigned char *);

int fin=0;
int AddIn_main(int isAppli, unsigned short OptionNum)
{
   
  Menu();
while (1)
{
    int sbs=0, max=100;
    unsigned char field[1024] = { 0 };
    PixEdit_SetImage(field);
    Config(&sbs,&max,field);
    Launch(sbs,max,field);
 }
  return 1;
}

void Menu()
{
    const unsigned char title[] = {
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
        0xc0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
        0x60,0x10,0x00,0x00,0x00,0x01,0xe0,0x00,0x00,0x00,0x00,0x00,
        0x78,0x38,0x3e,0x00,0x00,0x07,0xf0,0x00,0x00,0x00,0x00,0x00,
        0x78,0x38,0x7c,0x00,0x00,0x07,0xf8,0x00,0x00,0x00,0x00,0x00,
        0x78,0x38,0x7c,0x00,0x00,0x0f,0x78,0x00,0x00,0x00,0x00,0x00,
        0x78,0x38,0x70,0x00,0x00,0x0f,0x60,0x00,0x20,0x00,0x00,0x00,
        0x78,0x08,0x70,0x00,0x00,0x0f,0x00,0x00,0x10,0x00,0x00,0x00,
        0x78,0x20,0xf0,0x1f,0x00,0x0f,0x00,0x00,0x10,0xc6,0x07,0xc0,
        0x78,0x30,0xfe,0x3f,0x80,0x0f,0x78,0x00,0x3f,0xcf,0x0f,0xe0,
        0x78,0x38,0xfc,0x3f,0xc0,0x0f,0x78,0x7f,0x3f,0xff,0x0f,0xf0,
        0x78,0x38,0xfc,0x3b,0xc0,0x0e,0x78,0xff,0x3f,0xff,0x0e,0xf0,
        0x78,0x38,0xf8,0x7b,0xc0,0x0f,0x79,0xff,0x3d,0xe7,0x1e,0xf0,
        0x78,0x18,0xf0,0x7f,0xc0,0x0e,0x79,0xef,0x1d,0xe7,0x1f,0xf0,
        0x78,0x18,0xf0,0x7f,0xc0,0x0e,0x39,0xef,0x3c,0xe7,0x1f,0xf0,
        0x78,0x38,0x70,0x7e,0x00,0x0f,0x79,0xef,0x1c,0xe7,0x1f,0x80,
        0x38,0x38,0x70,0x38,0x40,0x0f,0x79,0xef,0x3c,0xe7,0x0e,0x10,
        0x7f,0xb8,0x70,0x3a,0xc0,0x0f,0xf9,0xef,0x3c,0xe7,0x0e,0xb0,
        0x7f,0x18,0x70,0x3b,0xc0,0x07,0xf9,0xff,0x1c,0xe7,0x0e,0xf0,
        0x7f,0x9c,0x70,0x3f,0x80,0x07,0xf9,0xff,0x1c,0x63,0x0f,0xe0,
        0x80,0x8c,0x70,0x1f,0x80,0x03,0xf8,0xf7,0x0c,0x63,0x87,0xe0,
        0x00,0x04,0x30,0x02,0x00,0x00,0x3c,0xc7,0x02,0x10,0x80,0x80,
        0x00,0x00,0x18,0x00,0x00,0x00,0x04,0x01,0x00,0x00,0x00,0x00,
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x00,0x00,0x00};
    const unsigned char press[] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 124, 249, 249, 227, 192, 243, 54, 97, 219, 246, 96, 50, 100, 203, 54, 97, 155, 54, 96, 241, 150, 96, 50, 100, 195, 6, 1, 155, 179, 192, 225, 131, 192, 60, 120, 241, 227, 193, 155, 241, 128, 193, 225, 128, 48, 112, 192, 48, 97, 251, 113, 128, 225, 129, 128, 48, 104, 203, 54, 97, 155, 49, 128, 241, 145, 128, 120, 237, 249, 227, 193, 155, 51, 193, 219, 243, 192, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
    const unsigned char dna[] = { 40, 16, 40, 124, 68, 254, 130, 254, 130, 124, 68, 56, 16, 56, 68, 124, 130, 254, 130, 254, 68, 124 };
    extern unsigned char *Disp_VRAM;
    unsigned int key;
    int c=0,f=0, i, n;
    
    Disp_Init();
    Disp_Clear();
    Disp_Bitmap(title,33,2,93,24,1);
    Disp_Bitmap(press,38,39,84,9,1);
    PrintMini(59,50,(const unsigned char *)"LePhenixNoir",0);
    
    
    while(1)
    {
        for(n=0;n<4;n++) for(i=0;i<64;i++) *(Disp_VRAM+(i<<4)+n) = dna[(i+(c*(5-n)))%22];
        if(c++==22) c=0;
        if(f++>5) f=5;
        
        if(f==5 && GetKeyWait(KEYWAIT_HALTOFF_TIMEROFF,-1,1,&key)==KEYREP_KEYEVENT) return;
        Bdisp_PutDisp_DD();
        Sleep(40);
    }
}
   
void Config(int *sbs, int *max, unsigned char *field)
{
  unsigned int key;
  char ch[20];
  int i,p=0;

  Disp_Clear();
  for(i=0;i<127;i++) Disp_Pixel(i,3,1);
  PrintXY(19,0,(const unsigned char *)" Configuration ",0);
    locate(3,3); Print((const unsigned char *)"Manuel:");
    locate(3,4); Print((const unsigned char *)"G\xE6\x0A""n\xE6\x0A""rations:");
  locate(3,5); Print((const unsigned char *)"Situation initale");
  locate(3,7); Print((const unsigned char *)"Lancer le jeu !");
  SaveDisp(2);

  Sleep(100);
  key = 0;

  while(1)
  {
    RestoreDisp(2);
    locate(11,3); Print((const unsigned char *)(*sbs?"Oui":"Non"));
    sprintf(ch,"%d",*max); locate(16,4); Print((const unsigned char *)ch);
    locate(2,p+3+(p==3)); Print((const unsigned char *)"\346\233");

    while(1)
    {
      GetKey(&key);

      if(key==KEY_CTRL_EXE) {
        if(!p) *sbs = !*sbs;
        else if (p==1) *max = getNum();
        else if (p==2) PixEdit_Edit();
        else if (p==3) return;
        break; }

      if(key==KEY_CTRL_UP && p)   { p--; break; }
      if(key==KEY_CTRL_DOWN && p-3) { p++; break; }
    }
  }
}

int getNum()
{
  char ch[5];

  PopUpWin(2);
  locate(3,3); Print((const unsigned char *)"G\xE6\x0A""n\xE6\x0A""rations");
  locate(3,4); Print((const unsigned char *)"[    ]");
  EI_input_string(ch,4,ALLOW_09,4,4,8,LEFT,NORMAL);

  return atoi(ch);
}

void Launch(int sbs, int max, unsigned char *field)
{

    extern unsigned char *Disp_VRAM;
  unsigned int key;
  int a,b, x,y;
  int i,v, n,o;

  Bkey_Set_RepeatTime(5,2);
  for(v=0;v<1024;v++) *(Disp_VRAM+v) = *(field+v);
    i=1;
    fin=0;
    while (fin==0)
  {
      for(v=0;v<1024;v++) *(field+v) = *(Disp_VRAM+v);

    if(sbs) {
      char ch[20];
      sprintf(ch,"G\xE6\x0A""n\xE6\x0A""ration %d",i);
      locate(1,8); Print((const unsigned char *)ch);
      GetKey(&key);
    }

    Bdisp_PutDisp_DD();

    for(v=0;v<1024;v++) *(Disp_VRAM+v) = *(field+v);

    for(y=1;y<63;y++) for(x=1;x<127;x++)
    {
      n = 0;
      o = PixEdit_GetPixel(x,y);
      for(a=-1;a<=1;a++) for(b=-1;b<=1;b++) if(a || b) n+=PixEdit_GetPixel(x+a,y+b);
      if(!o && n==3) Disp_Pixel(x,y,1);
      if(o && (n!=2 && n!=3)) Disp_Pixel(x,y,0);
    }
      if(IsKeyDown(KEY_CTRL_EXIT))fin=1;
      if(max!=0)
      {
          if(i>=max)fin=1;
          i++;
      }
  }

  if(!sbs) GetKey(&key);

  Bkey_Set_RepeatTime_Default();
}
#pragma section _BR_Size
unsigned long BR_Size;
#pragma section
#pragma section _TOP
int InitializeSystem(int isAppli, unsigned short OptionNum) {
    return INIT_ADDIN_APPLICATION(isAppli, OptionNum); }
#pragma section
/* AddIn_main(isAppli(1=MAIN,0=eAct), OptionNum(isAppli==0?0~3)) :: int(Done?1:0) */


